@@ -24,6 +24,8 @@ class VdexVocabulary(object):
         finally:
             f.close()
 
+        self.cache = {}
+
     def getTerms(self, lang):
         xpath = self.vdex.vdexTag('term')
         terms = self.vdex.tree._root.findall(xpath)
@@ -97,10 +99,14 @@ class VdexVocabulary(object):
         try:
             # TODO: need to depend only on zope
             from Products.CMFCore.utils import getToolByName
-            lang = getToolByName(context, 'portal_languages').getPreferredLanguage()
+            from zope.app.component.hooks import getSite
+            lang = getToolByName(getSite(), 'portal_languages').getPreferredLanguage()
         except:
             lang = None
 
+        if lang in self.cache:
+            return self.cache[lang]
+
         # build terms list
         items = self.getTerms(lang)
         relations = self.getRelations(context, lang, items)
@@ -112,7 +118,7 @@ class VdexVocabulary(object):
                 item['key'],
                 item['value'],
                 item['description'],
-                relations.get(item['key'], [])))
+                relations.get(item['key'], {})))
 
         # try to do ordering with zope.ucol support
         if not self.vdex.order_significant:
@@ -123,5 +129,6 @@ class VdexVocabulary(object):
             except:
                 terms.sort(key = lambda x: x.title)
 
-        return SimpleVocabulary(terms)
+        self.cache[lang] = SimpleVocabulary(terms)
+        return self.cache[lang]
 
